/*
 * A resource supporting the creation of user-specific ext2 filesystem opener
 * resources.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "ext2_file_opener.h"
#include "ext2_filesystem.h"
#include "resource_server.h"

Ext2Filesystem::Ext2Filesystem(Pages *pages, FileNotifierRegistry *notifiers, ext2_filsys fs)
: FilesystemResource(pages, notifiers)
{
    _ops = new Ext2FileOperations(fs);
}

Ext2Filesystem::~Ext2Filesystem()
{
    delete _ops;
}

/* Return a file opener object for the given user details. */

long Ext2Filesystem::open_for_user(ipc_varg_sys_uid_t uid, ipc_varg_sys_gid_t gid,
                                   ipc_varg_sys_mode_t umask, l4_cap_idx_t *ref)
{
    user_t user = (user_t) {uid.value, gid.value, umask.value};

    return open_for_user(user, ref);
}

/* Return a file opener object for the given user. */

long Ext2Filesystem::open_for_user(user_t user, l4_cap_idx_t *ref)
{
    Resource *resource = new Ext2FileOpener(this, _ops, user);

    /* Complete the initialisation and start a server in a new thread.
       If the thread does not start, the resource will be finalised. */

    return ResourceServer(resource).start_thread(ref);
}

// vim: tabstop=4 expandtab shiftwidth=4
